"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const trie_1 = require("./trie");
const util_1 = require("./util");
const suggest_1 = require("./suggest");
describe('Validate Trie Class', () => {
    it('Tests creating a Trie', () => {
        const trie = trie_1.Trie.create(sampleWords);
        chai_1.expect(trie).to.be.instanceof(trie_1.Trie);
    });
    it('Tests getting words from a Trie', () => {
        const trie = trie_1.Trie.create(sampleWords);
        chai_1.expect([...trie.words()]).to.be.deep.equal(sampleWords.sort());
    });
    it('Tests seeing if a Trie contains a word', () => {
        const trie = trie_1.Trie.create(sampleWords);
        chai_1.expect(trie.has('lift')).to.be.true;
        chai_1.expect(trie.has('fork-lift')).to.be.false;
    });
    it('Tests complete', () => {
        const trie = trie_1.Trie.create(sampleWords);
        chai_1.expect([...trie.completeWord('lift')]).to.be.deep.equal(sampleWords.filter(w => w.slice(0, 4) === 'lift').sort());
        chai_1.expect([...trie.completeWord('life')]).to.be.deep.equal([]);
        chai_1.expect([...trie.completeWord('lifting')]).to.be.deep.equal(['lifting']);
    });
    it('Tests insert', () => {
        const trie1 = trie_1.Trie.create(sampleWords);
        const trie2 = trie_1.Trie.create([]);
        sampleWords.forEach(word => trie2.insert(word));
        util_1.orderTrie(trie2.root);
        const words1 = [...trie1.words()];
        const words2 = [...trie2.words()];
        chai_1.expect(words2).to.be.deep.equal(words1);
    });
    it('tests suggestions', () => {
        const trie = trie_1.Trie.create(sampleWords);
        const suggestions = trie.suggest('wall', 10);
        chai_1.expect(suggestions).to.contain('walk');
    });
    it('tests genSuggestions', () => {
        const trie = trie_1.Trie.create(sampleWords);
        const collector = suggest_1.suggestionCollector('wall', 10);
        trie.genSuggestions(collector);
        chai_1.expect(collector.suggestions.map(a => a.word)).to.contain('walk');
    });
    it('Tests iterate', () => {
        const trie = trie_1.Trie.create(sampleWords);
        const words = [...trie.iterate()]
            .filter(r => util_1.isWordTerminationNode(r.node))
            .map(r => r.text);
        chai_1.expect(words).to.be.deep.equal(sampleWords.sort());
    });
    it('Test where only part of the word is correct', () => {
        const trie = trie_1.Trie.create(sampleWords);
        chai_1.expect(trie.has('talking')).to.be.true;
        chai_1.expect(trie.has('talkings')).to.be.false;
    });
    it('Test compound words', () => {
        // cspell:ignore talkinglift joywalk jwalk
        const trie = trie_1.Trie.create(sampleWords);
        chai_1.expect(trie.has('talkinglift', true)).to.be.true;
        chai_1.expect(trie.has('joywalk', true)).to.be.true;
        chai_1.expect(trie.has('jaywalk', true)).to.be.true;
        chai_1.expect(trie.has('jwalk', true)).to.be.false;
        chai_1.expect(trie.has('joywalk', false)).to.be.false;
        chai_1.expect(trie.has('walked', true)).to.be.true;
    });
});
const sampleWords = [
    'edit',
    'end',
    'edge',
    'walk',
    'walked',
    'walker',
    'walking',
    'walks',
    'talk',
    'talks',
    'talked',
    'talker',
    'talking',
    'lift',
    'lifts',
    'lifted',
    'lifter',
    'lifting',
    'jay',
    'journal',
    'journals',
    'journalism',
    'journalist',
    'journalistic',
    'journey',
    'journeyer',
    'journeyman',
    'journeymen',
    'joust',
    'jouster',
    'jousting',
    'jovial',
    'joviality',
    'jowl',
    'jowly',
    'joy',
    'joyful',
    'joyfuller',
    'joyfullest',
    'joyfulness',
    'joyless',
    'joylessness',
    'joyous',
    'joyousness',
    'joyridden',
    'joyride',
    'joyrider',
    'joyriding',
    'joyrode',
    'joystick',
];
//# sourceMappingURL=trie.test.js.map